#!/usr/bin/perl

BEGIN { push (@INC,'/qos/modules'); }
use qos_config ;
undef $DEBUG ;
#$DEBUG = yes ;
use RRDs ;

# If we have to use snmp to get the data, set the snmp variable
if ( $ARGV[0] eq "snmp" ) {
	$snmp = "yes" ;
}

# We need to open an snmp session to get the data
if ( $snmp ) {
	use SNMP ;
	$snmp_session = new SNMP::Session(DestHost => $host, Community => $community, Version => "2" );

	die "die opening session $snmp_session->{ErrorStr}\n" if $snmp_session->{ErrorStr} ; # Exit if there was an error
	print "host : $host, Community : $community\n" if $DEBUG ;
}

# Small sub function to get some data.
sub get_value {
	$request = $_[0] ;
	# Use snmp
	if ($snmp) {
		$return = $snmp_session->get($request) ;
		 die "die getting data : $snmp_session->{ErrorStr}\n" if $snmp_session->{ErrorStr} ; # Exit if there was an error
	# Ugly but it works
	} else {
		@return = `/qos/bin/snmp_tc.pl -g $request` ;
		$return = ( split "\n", `/qos/bin/snmp_tc.pl -g $request`)[2] ;
	}
	print "Request : $request\n" if $DEBUG ;
	print "Return : $return\n" if $DEBUG ;
	return $return ;
}

# Get the number of nics
$nics = get_value ("$oid.0") ;

# Query the class/qdiscs for each nic
$nic = 0 ;
while ($nic < $nics ) {
	$nic ++ ;
	$nic{$nic} = get_value ("$oid.0.$nic") ;
	print "NIC : $nic $nic{$nic}\n" if $DEBUG ;
	@childs = split " " , get_value ("$oid.1.$nic") ;
	foreach $child (@childs) {
		find_childs( $child) ;
	}
}

sub find_childs {
	$parent = $_[0] ;
	@split = split ":", $parent ;
	$split[0] = hex $split[0] ;
	$split[1] = hex $split[1] ;

	local @childs = split " ", get_value ("$oid.2.$nic.$split[0].$split[1]") ;
	$data->{$parent}{bytes} = get_value ("$oid.5.$nic.$split[0].$split[1]") ;

	if ( ( $#childs eq "-1" ) or ( $childs[0] =~ /NOSUCHINSTANCE/ ) ) {
		$data->{$parent}{leaf} = get_value ("$oid.4.$nic.$split[0].$split[1]") ;
		$leaf = hex $leaf ;
		$data->{$parent}{type} = get_value ("$oid.3.$nic.$leaf") ;
	} else {
		$data->{$parent}{leaf} = "NONE" ;
		$data->{$parent}{type} = "NONE" ;
		@{$data->{$parent}{childs}} = @childs ;

		foreach $child ( @childs ) {
			find_childs ($child) ;
		}
	}
}

# Update the rrd files with the counters we found
foreach $class (keys(%{$data})) {
	$temp = $class ;
	$temp =~  s/:/_/g ;
	$rrd = $RRDS . "/" . $nic{$nic}  . "." . $temp ;

	local @temp = split " ", $data->{$class}{bytes} ;
	update_rrd ($rrd,@temp) ;
}

# Uncomment the next 2 lines for debug purpose
#use Data::Dumper ;
#print Dumper ($data) ;

# Update rrd
# Called for each qdisc/class and each interface
sub update_rrd {
	$date = `date +%s` ;
	@input = @_ ;
	$rrd_base = shift @input ;

	$rrd = $rrd_base . ".1.rrd" ;
	if ( ! -f $rrd ) {
		RRDs::create ($rrd,
			"DS:by:DERIVE:600:0:U",
			"DS:pa:DERIVE:600:0:U",
			"DS:dr:DERIVE:600:0:U",
			"DS:ov:DERIVE:600:0:U",
			"RRA:MIN:0.5:1:576","RRA:MIN:0.5:6:576","RRA:MIN:0.5:24:576","RRA:MIN:0.5:288:576",
			"RRA:MAX:0.5:1:576","RRA:MAX:0.5:6:576","RRA:MAX:0.5:24:576","RRA:MAX:0.5:288:576",
			"RRA:AVERAGE:0.5:1:576","RRA:AVERAGE:0.5:6:576","RRA:AVERAGE:0.5:24:576","RRA:AVERAGE:0.5:288:576"
		) ;
		$err = RRDs::error;
		print "Error while creating $rrd : $err\n" if $err ;
	}
	print "Updating $rrd\n  with $input[0]:$input[1]:$input[2]:$input[3]\n" if $DEBUG ;
	RRDs::update ("$rrd","$date:$input[0]:$input[1]:$input[2]:$input[3]");
	$err = RRDs::error;
	print "Error while updating $rrd : $err\n" if $err ;

	if ( $input[4] ne "NONE" ) {
		$rrd = $rrd_base . ".2.rrd" ;
		if ( not -f $rrd ) {
			RRDs::create ($rrd,
				"DS:le:DERIVE:600:0:U",
				"DS:bo:DERIVE:600:0:U",
				"DS:gi:DERIVE:600:0:U",
				"RRA:MIN:0.5:1:576","RRA:MIN:0.5:6:576","RRA:MIN:0.5:24:576","RRA:MIN:0.5:288:576",
				"RRA:MAX:0.5:1:576","RRA:MAX:0.5:6:576","RRA:MAX:0.5:24:576","RRA:MAX:0.5:288:576",
				"RRA:AVERAGE:0.5:1:576","RRA:AVERAGE:0.5:6:576","RRA:AVERAGE:0.5:24:576","RRA:AVERAGE:0.5:288:576") ;
			$err = RRDs::error;
			print "Error while creating $rrd : $err\n" if $err ;
		}
		print "Updating $rrd\n  with $input[4]:$input[5]:$input[6]\n" if $DEBUG ;
		RRDs::update ("$rrd","$date:$input[4]:$input[5]:$input[6]");
		$err = RRDs::error;
		print "Error while updating $rrd : $err\n" if $err ;
	}

	if ( $input[8] ne "NONE" ) {
		$rrd = $rrd_base . ".3.rrd" ;
		if ( not -f $rrd ) {
			RRDs::create ($rrd,
				"DS:to:DERIVE:600:0:U",
				"DS:ct:DERIVE:600:0:U",
				"RRA:MIN:0.5:1:576","RRA:MIN:0.5:6:576","RRA:MIN:0.5:24:576","RRA:MIN:0.5:288:576",
				"RRA:MAX:0.5:1:576","RRA:MAX:0.5:6:576","RRA:MAX:0.5:24:576","RRA:MAX:0.5:288:576",
				"RRA:AVERAGE:0.5:1:576","RRA:AVERAGE:0.5:6:576","RRA:AVERAGE:0.5:24:576","RRA:AVERAGE:0.5:288:576") ;
			$err = RRDs::error;
			print "Error while creating $rrd : $err\n" if $err ;
		}
		print "Updating $rrd\n  with $input[7]:$input[8]\n" if $DEBUG ;
		RRDs::update ("$rrd","$date:$input[7]:$input[8]");
		$err = RRDs::error;
		print "Error while updating $rrd : $err\n" if $err ;
	}
}
