#!/usr/bin/perl
#
# Use at own risk
# Version 0.2
# HEB 3.6 has no injects anymore
#
# Version 0.1
# Clean ups
#
# Version 0.1
#
# Stef Coene <stef.coene@docum.org>
#
# This script must be run each 5 minutes to update the rrd databases
#
# If you provide snmp as parameter, it will query a snmp server.  The needed info (hosts, OID, ...) can be specified in qos_config.pm.
#     Don't forget to change the config to relect your situation.
# Otherwise it will execute the script snmp.pl locally so you don't have to run a snmp-server.

BEGIN { push (@INC,'/qos/modules'); }
use qos_config ;
undef $DEBUG ;
use RRDs ;
use CGI qw (:standard) ;

# Check for the rrd-locatie directory
if ( ! -d $RRDS ) {
	print "$RRDS : directory not found, Exitting\n" ;
	exit ;
}

# Get info
if ( $ARGV[0] eq "snmp" ) {
	use SNMP ;
	$obj = new SNMP::Session DestHost, $host , Community, $community ;
	$result = $obj->get([[$oid]]) ;
	die "die $obj->{ErrorStr}\n" if $obj->{ErrorStr} ; # Exit if there was an error
} else {
	$result = `$BIN/snmp.pl` ;
	$result =~ s/\"//g ;
	$result =~ s/=//g ;
}
@result = split " ", $result ;

# Find the total of classes/qdiscs returned so we can process them one by one
$total = ( $#result  + 1 ) / 12 -1 ;

# Create some handy variables :
foreach $i (0 .. $total) {
	$int                     = $result [ $i * 12      ] ;
	$key                     = $result [ $i * 12 + 1  ] ;
	if ( defined $DEBUG ) { print "Testing received data : interface = $int   as $i\n" ; }
	$parent{"$int $key"}     = $result [ $i * 12 + 2  ] ;
	$bytes{"$int $key"}      = $result [ $i * 12 + 3  ] ;
	$pkts{"$int $key"}       = $result [ $i * 12 + 4  ] ;
	$dropped{"$int $key"}    = $result [ $i * 12 + 5  ] ;
	$overlimits{"$int $key"} = $result [ $i * 12 + 6  ] ;
	$lended{"$int $key"}     = $result [ $i * 12 + 7  ] ;
	$borowwed{"$int $key"}   = $result [ $i * 12 + 8  ] ;
	$giants{"$int $key"}     = $result [ $i * 12 + 9  ] ;
	$tokens{"$int $key"}     = $result [ $i * 12 + 10 ] ;
	$ctokens{"$int $key"}    = $result [ $i * 12 + 11 ] ;
}

$date = `date +%s` ;

# Create a page with an overview of all handles we found
# Refresh om de 300 seconden (5 minuten)
open (HTML, ">$html_output") || die "Can not open $html_output for writing.\n";
print HTML qq{<html><head>
<META HTTP-EQUIV="REFRESH" CONTENT="300">
<META HTTP-EQUIV="EXPIRES" CONTENT="Sat, 01 Jan 2001 00:00:00 GMT">
<link rel="stylesheet" type="text/css" href="$CSS" />
</head><body>
} ;
print HTML `date` ;

# Find all children for a parent
# Used to stack values of classes who have the same parent
foreach $key (keys(%parent)) {
	if ( $parent{$key} eq "NONE" ) { next ; }  # the only class with no parent is the root class
	($int,$handle) = split(" ",$key) ;
	$test = $int . "_" . $parent{$key} ; # "parent" key
	$child{$test} .= " $handle " ; # %child is a list of childs with the same parent
}

# Create stackable graphs for classes who share the same parent
foreach $key (keys(%child)) {
	($int,$parent) = split ("_",$key) ;
	@handle = split (" ",$child{$key}) ; # This is a list of child classes
	if ( $#handle > 0 ) { # If we have more then 1 childs class, we can stack them
		@handle = sort (@handle) ;
		$temp = join ("_",@handle) ;
		$handles = join (" ",@handle) ;
		print HTML qq{<H1>Handle $parent on interface $int with childs $handles</H1>};
		print HTML qq{<p><A HREF="$CGI_BASE/tcrrd.pl?handle=$parent&int=$int&childs=$temp"><IMG SRC=$CGI_BASE/tcrrd.pl?handle=$parent&int=$int&childs=$temp&type=bytes&graph=1 BORDER=0></A>
} ;
	}
}

# Proces all other classes
foreach $key (sort (keys(%bytes))) {
	@split = split " ", $key ;
	$int = $split [0] ;
	$handle = $split [1] ;
	$temp = $handle ;
	$handle =~ s/_/:/g ;
	print HTML qq{<H1>Handle $handle on interface $int</H1>};
	print HTML qq{<p><A HREF="$CGI_BASE/tcrrd.pl?handle=$temp&int=$int&parent=$parent"><IMG SRC=$CGI_BASE/tcrrd.pl?handle=$temp&int=$int&type=bytes&graph=1 BORDER=0></A>
} ;
	$rrd = $RRDS . "/" . $int . "." . $temp ;
	$rrd =~ s/:/_/g ;
	update_rrd ($rrd,$bytes{$key},$pkts{$key},$dropped{$key},$overlimits{$key},$lended{$key},$borowwed{$key},$giants{$key},$tokens{$key},$ctokens{$key}); 
}

exit ;
print HTML qq{</body></html>};
close HTML;


# Update rrd
# Called for each qdisc/class and each interface
sub update_rrd () {
	@input = @_ ;
	$rrd_base = shift @input ;

	$rrd = $rrd_base . ".1.rrd" ;
	if ( ! -f $rrd ) {
		RRDs::create ($rrd,
			"DS:by:DERIVE:600:0:U",
			"DS:pa:DERIVE:600:0:U",
			"DS:dr:DERIVE:600:0:U",
			"DS:ov:DERIVE:600:0:U",
			"RRA:MIN:0.5:1:576","RRA:MIN:0.5:6:576","RRA:MIN:0.5:24:576","RRA:MIN:0.5:288:576",
			"RRA:MAX:0.5:1:576","RRA:MAX:0.5:6:576","RRA:MAX:0.5:24:576","RRA:MAX:0.5:288:576",
			"RRA:AVERAGE:0.5:1:576","RRA:AVERAGE:0.5:6:576","RRA:AVERAGE:0.5:24:576","RRA:AVERAGE:0.5:288:576"
		) ;
		$err = RRDs::error;
		print "Error while creating $rrd : $err\n" if $err ;
	}
	print "Updating $rrd\n  with $input[0]:$input[1]:$input[2]:$input[3]\n" if $DEBUG ;
	RRDs::update ("$rrd","$date:$input[0]:$input[1]:$input[2]:$input[3]");
	$err = RRDs::error;
	print "Error while updating $rrd : $err\n" if $err ;

	if ( $input[4] ne "NONE" ) {
		$rrd = $rrd_base . ".2.rrd" ;
		if ( not -f $rrd ) {
			RRDs::create ($rrd,
				"DS:le:DERIVE:600:0:U",
				"DS:bo:DERIVE:600:0:U",
				"DS:gi:DERIVE:600:0:U",
				"RRA:MIN:0.5:1:576","RRA:MIN:0.5:6:576","RRA:MIN:0.5:24:576","RRA:MIN:0.5:288:576",
				"RRA:MAX:0.5:1:576","RRA:MAX:0.5:6:576","RRA:MAX:0.5:24:576","RRA:MAX:0.5:288:576",
				"RRA:AVERAGE:0.5:1:576","RRA:AVERAGE:0.5:6:576","RRA:AVERAGE:0.5:24:576","RRA:AVERAGE:0.5:288:576") ;
			$err = RRDs::error;
			print "Error while creating $rrd : $err\n" if $err ;
		}
		print "  Updating $rrd\n    with $input[4]:$input[5]:$input[6]\n" if $DEBUG ;
		RRDs::update ("$rrd","$date:$input[4]:$input[5]:$input[6]");
		$err = RRDs::error;
		print "Error while updating $rrd : $err\n" if $err ;
	}

	if ( $input[8] ne "NONE" ) {
		$rrd = $rrd_base . ".3.rrd" ;
		if ( not -f $rrd ) {
			RRDs::create ($rrd,
				"DS:to:DERIVE:600:0:U",
				"DS:ct:DERIVE:600:0:U",
				"RRA:MIN:0.5:1:576","RRA:MIN:0.5:6:576","RRA:MIN:0.5:24:576","RRA:MIN:0.5:288:576",
				"RRA:MAX:0.5:1:576","RRA:MAX:0.5:6:576","RRA:MAX:0.5:24:576","RRA:MAX:0.5:288:576",
				"RRA:AVERAGE:0.5:1:576","RRA:AVERAGE:0.5:6:576","RRA:AVERAGE:0.5:24:576","RRA:AVERAGE:0.5:288:576") ;
			$err = RRDs::error;
			print "Error while creating $rrd : $err\n" if $err ;
		}
		print "    Updating $rrd\n      with $input[7]:$input[8]\n" if $DEBUG ;
		RRDs::update ("$rrd","$date:$input[7]:$input[8]");
		$err = RRDs::error;
		print "Error while updating $rrd : $err\n" if $err ;
	}
}
