#!/usr/bin/perl

# x = nic
# y = major number
# z = major number (optional)
#   y and z are in decimals !!!!
#
# Number of NIC's
# snmpwalk localhost public .1.3.6.1.4.1.2021.257.0
#
# Name of NIC
# snmpwalk localhost public .1.3.6.1.4.1.2021.257.0.x
#
# Child qdisc 
# snmpwalk localhost public .1.3.6.1.4.1.2021.257.1.x
#
# Class info
# snmpwalk localhost public .1.3.6.1.4.1.2021.257.2.x.y.z
#
# Qdisc type
# snmpwalk localhost public .1.3.6.1.4.1.2021.257.3.x.y

BEGIN { push (@INC,'/qos/modules'); }

use Data::Dumper ;
use qos_config ;
use qos ;

$request="$ARGV[1]" ;
# Debug
`echo ARGV @ARGV >> /tmp/debug` ;

# Find a list of network cards
%nic = nics () ;

@request = split ('\.', $request) ;

if ( $request eq $oid ) {
	#print $oid . "\.0.0\n" ;
	print "$oid\n" ;
	print "string\nNetwork interface names\n" ;

# Returning NIC information
} elsif ( $request =~ /$oid\.0/   ) {
	if ( $request =~ /$oid\.0\.\d/   ) {
		if ( $nic{$request[-1]} ) {
			print "$request\n" ;
			print "string\n$nic{$request[-1]}\n" ;
		#} else {
			#print $oid . "\.1.0\n" ;
			#print "string\nRoot classes\n" ;
		}
	} else {
		print "$request\n" ;
		@temp = keys (%nic) ;
		$#temp ++ ;
		print "string\n$#temp\n" ;
	}

# Returning the classes attached to the root qdisc
} elsif ( $request =~ /$oid\.1\.\d/   ) {
	if ( $nic{$request[-1]} ) {
		read_config_tc_tree ($nic{$request[-1]}) ;
		print "$request\n" ;
		print "string\n$tree->{root}{childs}\n" ;
	# } else {
		#print $oid . "\.2.2.1\n" ;
		#print "string\nChild information\n" ;
	}

# Returning the child classes
} elsif ( $request =~ /$oid\.[2345]\./ ) {
	$minor = lc(sprintf'%X', $request[-1] ); # Convert to hex

	# 2 possible requests :
	# snmpwalk localhost public .1.3.6.1.4.1.2021.257.2.x.y.z
	# snmpwalk localhost public .1.3.6.1.4.1.2021.257.2.x.y
	if ( $request =~ /$oid\.[2345]\.[0-9]+\.[0-9]+\.[0-9]+/ ) { 
		$request[-2] = lc(sprintf'%X', $request[-2] );
		$class = $request[-2] . ":" . $minor ;
		$nic = $request[-3] ;
	} else {
		$class = $minor . ":" ;
		$nic = $request[-2] ;
	}
#`echo $request >> /tmp/debug ` ;
	read_config_tc_tree ($nic{$nic}) ;
#`echo NIC $nic $nic{$nic} >> /tmp/debug ` ;

	if ( $request =~ /$oid\.2/ ) {
		if ( $tree->{$class}{childs} ) {
			print "$request\n" ;
			print "string\n$tree->{$class}{childs}\n" ;
			#`echo CHILDS $tree->{$class}{childs} >> /tmp/debug ` ;
		}
	} elsif ( $request =~ /$oid\.3/ ) {
		# `echo TYPE $class $tree->{$class}{type} >> /tmp/debug ` ;
		print "$request\n" ;
		if ( $tree->{$class}{type} ) {
			print "string\n$tree->{$class}{type}\n" ;
		} else {
			print "string\nNONE\n" ;
		}
	} elsif ( $request =~ /$oid\.4/ ) {
		# `echo LEAF $class $tree->{$class}{leaf} >> /tmp/debug ` ;
		print "$request\n" ;
		if ( $tree->{$class}{leaf} ) {
			print "string\n$tree->{$class}{leaf}\n" ;
		} else {
			print "string\nNONE\n" ;
		}
	} else {
		#`echo STAT $class >> /tmp/debug ` ;
		%counters = find_counters ($nic{$nic}) ;
		print "$request\n" ;
		print "string\n$counters{$class}\n" ;
	}
}

sub find_counters {
	$nic = $_[0] ;
	@qdisc = `$TC -s -d qdisc show dev $nic` ; # Get all qdisc info
	@class = `$TC -s -d class show dev $nic` ; # Get all class info

	foreach $ele (@qdisc) {
		#  qdisc sfq 8045: quantum 1514b limit 128p flows 128/1024
		$ele =~ /\s+([A-Za-z0-9]+:)\s+/o and $number = $1 ;
		# Sent 0 bytes 0 pkts (dropped 0, overlimits 0)
		$ele =~ /Sent\s+(\d+)\s+bytes\s+(\d+)\s+pkts\s+\(dropped\s+(\d+),\s+overlimits\s+(\d+)\)/
			and $bytes{"$number"} = "$1 $2 $3 $4"
			and next ;
	}

	foreach $ele (@class) {
		chomp ($ele) ;
		#class htb 1:1 root prio 0 rate 16000Kbit ceil 16000Kbit burst 2Kb/8 mpu 0b cburst 22077b/8 mpu 0b quantum 60000 level 3
		# If we have a root class, the parent = "major_number:"
		$ele =~ /\s+([A-Za-z0-9]+:[A-Za-z0-9]+)\s+root/o
			and $number = $1 ;

		#class htb 1:100 parent 1:10 leaf 8002: prio 0 rate 8000Kbit ceil 8000Kbit burst 2Kb/8 mpu 0b cburst 11838b/8 mpu 0b quantum 60000 level 0
		$ele =~ /\s+([A-Za-z0-9]+:[A-Za-z0-9]+)\s+parent\s+([A-Za-z0-9]+:[A-Za-z0-9]+)\s+leaf\s+([A-Za-z0-9]+:)\s+/o
			and $number = $1
			and next ;

		#class htb 1:10 parent 1:1 prio 0 rate 8000Kbit ceil 8000Kbit burst 2Kb/8 mpu 0b cburst 11838b/8 mpu 0b quantum 60000 level 2
		$ele =~ /\s+([A-Za-z0-9]+:[A-Za-z0-9]+)\s+parent\s+([A-Za-z0-9]+:[A-Za-z0-9]+)\s+/o
			and $number = $1
			and next ;

		#Sent 0 bytes 0 pkts (dropped 0, overlimits 0)
		$ele =~ /Sent\s+(\d+)\s+bytes\s+(\d+)\s+pkts\s+\(dropped\s+(\d+),\s+overlimits\s+(\d+)\)/
			and $bytes{"$number"} = "$1 $2 $3 $4"
			and next ;

		# lended: 0 borrowed: 0 giants: 0
		# lended: 0 borrowed: 0 giants: 0 injects: 0
		$ele =~ /lended:\s+(\d+)\s+borrowed:\s+(\d+)\s+giants:\s+(\d+)/
			and $lended{"$number"} = "$1 $2 $3"
			and next ;

		#  tokens: 1637 ctokens: 9471
		$ele =~ /tokens:\s+(\d+)\s+ctokens:\s+(\d+)/
			and $tokens{"$number"} = "$1 $2"
			and next ;
	}

	$return{$class} = "$bytes{$class} " ;
	if ( defined $lended{$class} ) {
		$return{$class} = $return{$class} . "$lended{$class} " ;
	} else {
		$return{$class} = $return . " NONE NONE NONE " ;
	}
	if ( defined $tokens{$class} ) {
		$return{$class} = $return{$class} . "$tokens{$class} " ;
	} else {
		$return{$class} = $return{$class} . "NONE NONE " ;
	}
	return %return ;
}
